from __future__ import print_function

import os.path

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

class google_connector():
	def __init__(self, **kwargs):
		"""
			Build your connector object. Gdrive function and Gsheet function activate.
		"""
		SCOPES = ['https://www.googleapis.com/auth/drive','https://www.googleapis.com/auth/spreadsheets']
		self.creds = None
		# The file token.json stores the user's access and refresh tokens, and is
		# created automatically when the authorization flow completes for the first
		# time.
		if os.path.exists('token.json'):
			self.creds = Credentials.from_authorized_user_file('token.json', SCOPES)
		# If there are no (valid) credentials available, let the user log in.
		if not self.creds or not self.creds.valid:
			if self.creds and self.creds.expired and self.creds.refresh_token:
				self.creds.refresh(Request())
			else:
				flow = InstalledAppFlow.from_client_secrets_file('credentials.json', SCOPES)
				self.creds = flow.run_local_server(port=0)
		# Save the credentials for the next run
		with open('token.json', 'w') as token:
			token.write(self.creds.to_json())

	def show_folders(self, *args):
		"""
			Return list of file from your root drive if no argument.
			Return list of file from the drive repertory passed in first arguements
		"""
		service = build('drive', 'v3', credentials=self.creds)
		if len(args):
			query = f"'{args[1]}' in parents"
		else:
			query = None
		results = service.files().list(q=query, pageSize=999, fields="nextPageToken, files(id, name)").execute()
		items = results.get('files', [])
		return (items)

	def create_folders(self, name, *args):
		try:
		# create drive api client
			service = build('drive', 'v3', credentials=self.creds)
			file_metadata = {
			'name': name,
			'mimeType': 'application/vnd.google-apps.folder',
			}
			if len(args) > 0:
				file_metadata.update({
					'parents' : [args[1]]
					})
			file = service.files().create(body=file_metadata, fields='id'
			).execute()
			print(F'Folder ID: "{file.get("id")}".')
			return file.get('id')
		except HttpError as error:
			print(F'An error occurred: {error}')

	def copy_file(self, name, file_id, parent_id):
		try:
		# create drive api client
			service = build('drive', 'v3', credentials=self.creds)
			file_metadata = {
			'name': name,
			'parents': parent_id,
			'starred': False,
			}
			file = service.files().copy(body=file_metadata, fileId=file_id).execute()
			return file.get('id')
		except HttpError as error:
			print(F'An error occurred: {error}')

	def update_values(self, spreadsheet_id, cell_range, cell_values):
		try:
			service = build('sheets', 'v4', credentials=self.creds)
			data = {
			'values': cell_values
			}
			result = service.spreadsheets().values().update(
			spreadsheetId=spreadsheet_id,
			range=cell_range,
			valueInputOption='USER_ENTERED',
			body=data
			).execute()
			return result
		except HttpError as error:
			print(f"An error occurred: {error}")
			return error





def build_ret(elem):
	ret = {
		'type' : elem['type'],
		'properties' : elem,
		'update' : False,
		'content' : False
	}
	return (ret)

def ret_status(elem):
	ret = build_ret(elem)
	ret.update({
		'content' : True,
		'color' : elem['status']['color'],
		'name' : elem['status']['name']
	})
	return ret

def ret_date(elem):
	ret = build_ret(elem)
	if elem['date'] != None:
		ret.update({
			'content' : True,
			'start' : elem['date']['start'],
			'end' : elem['date']['end']
		})
	return ret

def ret_formula(elem):
	ret = build_ret(elem)
	if elem['formula'] != None:
		ret.update({
			'content' : True,
			'type' : 'number',
			'number' : elem['formula']['number']
		})
	return ret

def ret_rich_text(elem):
	ret = build_ret(elem)
	if elem['rich_text'] != []:
		ret.update({
			'content' : True,
			'rich_text' : elem['rich_text'][0]['plain_text']
		})
	return ret

def ret_title(elem):
	ret = build_ret(elem)
	if elem['title'] != []:
		ret.update({
			'content' : True,
			'title' : elem['title'][0]['plain_text']
		})
	return ret

def ret_number(elem):
	ret = build_ret(elem)
	ret.update({
		'content' : True,
		'number' : elem['number']
	})
	return ret

def ret_select(elem):
	ret = build_ret(elem)
	if elem['select'] != None:
		ret.update({
			'content' : True,
			'color' : elem['select']['color'],
			'name' : elem['select']['name']
		})
	return ret

def ret_multi_select(elem):
	ret = build_ret(elem)
	ret.update({
		'content' : True,
		'multi_select' : []
	})
	for select in elem['multi_select']:
		data = {
		'color' : select['color'],
		'name' : select['name']
		}
		ret['multi_select'].append(data)
	return ret

def ret_checkbox(elem):
	ret = build_ret(elem)
	ret.update({
		'content' : True,
		'checkbox' : elem['checkbox']
	})
	return ret

def ret_people(elem):
	ret = build_ret(elem)
	ret.update({
		'content' : True,
		'people' : []
	})
	for people in elem['people']:
		ret['people'].append(people['name'])
	return ret

def ret_url(elem):
	ret = build_ret(elem)
	if elem['url'] != None:
		ret.update({
			'content' : True,
			'url' : elem['url']
		})
	return ret

def ret_created_time(elem):
	ret = build_ret(elem)
	ret.update({
		'content' : True,
		'created_time' : elem['created_time']
	})
	return ret

def ret_created_by(elem):
	ret = build_ret(elem)
	ret.update({
		'content' : True,
		'creator_id' : elem['created_by']['id'],
		'creator_name' : elem['created_by']['name']
	})
	return ret

def ret_last_edited_time(elem):
	ret = build_ret(elem)
	ret.update({
		'content' : True,
		'last_edited_time' : elem['last_edited_time']
	})
	return ret

def ret_last_edited_by(elem):
	ret = build_ret(elem)
	ret.update({
		'content' : True,
		'editor_id' : elem['last_edited_by']['id'],
		'editor_name' : elem['last_edited_by']['name']
	})
	return ret

def ret_email(elem):
	ret = build_ret(elem)
	if elem['email'] != None:
		ret.update({
			'content' : True,
			'email' : elem['email']
		})
	return ret

def ret_phone_number(elem):
	ret = build_ret(elem)
	if elem['phone_number'] != None:
		ret.update({
			'content' : True,
			'phone_number' : elem['phone_number']
		})
	return ret


def get_properties():
	ret = {
		"status" : ret_status,
		"date" : ret_date,
		"rich_text" : ret_rich_text,
		"title" : ret_title,
		"number" : ret_number,
		"select" : ret_select,
		"people" : ret_people,
		"checkbox" : ret_checkbox,
		"url" : ret_url,
		"email" : ret_email,
		"phone_number" : ret_phone_number,
		"multi_select" : ret_multi_select,
		"created_time" : ret_created_time,
		"created_by" : ret_created_by,
		"last_edited_time" : ret_last_edited_time,
		"last_edited_by" : ret_last_edited_by,
		"formula" : ret_formula
	}
	return ret



from setuptools import setup

setup(name='ftThanos',
version="0.0.50",
description="A client for 50 API",
author="Thanos 50",
author_email="thanos@50paris.fr",
packages=["ftThanos"],
install_requires=["requests","python-dotenv"],
)
